using System;
using System.IO;
using System.Xml;
using System.Data;
using System.Drawing;
using System.Windows.Forms;
using Library;

public struct CodeSample
{
	public string Title;
	public string Description;
	public string Language;
	public string Search;
	public string Topic;
	public string Code;
	public string Usage;
}
public struct Language
{
	public string Name;
	public string Alias;
	public Color Colour;
	public string FileName;
	public DataSet File;
	public int Icon;
	public bool Split;
	public string Lang1;
	public string Lang2;
}
public struct Topic
{
	public string Name;
	public string Alias;
	public int Icon;
	public int NumSamples;
	public int CurrNum;
	public CodeSample[] Samples;
}


class modMain
{
	public const int SEARCH = 0;
	public const int INDEX = 0;
	public static fMain frmMain;
	public static fAdd frmAdd;
	public static fSearch frmSearch;
	public static fCode frmCode;
	public static bool Finish = false;
	public static string AppName;
	public static Point frmPos;
	public static bool ShowEmptyFolders;
	public static ImageList iconList = new ImageList();
	public static string[] SearchedFor;
	public static string SaveSearched;

	private static XmlDocument Settings = new XmlDocument();
	public static Language[] Languages;
	public static Topic[] Topics;
	private static string[] Icons;
	public static CodeSample[] PreviousSearch;
	private static int iconCount = -1;
	private static NotifyIcon Tray = new NotifyIcon();
	public static CodeSample SelectedCode;
	private static string[] OpenCodeWindows;
	private static TreeNode[] EmptyNodes;

	static void Main()
	{
		EmptyNodes = new TreeNode[0];
		OpenCodeWindows = new String[0];
		PreviousSearch = new CodeSample[0];
		LoadAppSettings();
		frmAdd = new fAdd();
		frmMain = new fMain();
		frmAdd.Icon = frmMain.Icon;
		frmMain.Show();
		LoadSystemTrayIcon();
		SetupIcons();
		LoadLanguages();
		LoadTopics();
		LoadCode();
		//RemoveEmptyFolders();
		do
		{
			Application.DoEvents();
		}	while (Finish == false);
		Tray.Dispose();
		SaveAppSettings();
	}

	public static void OpenCodeWindow()
	{
		foreach(string str in OpenCodeWindows)
			if(str == SelectedCode.Language + ": " + SelectedCode.Title)
				return;
		string[] NewArray;
		NewArray = new string[OpenCodeWindows.Length + 1];
		OpenCodeWindows.CopyTo(NewArray,0);
		NewArray[NewArray.Length - 1] = SelectedCode.Language + ": " + SelectedCode.Title;
		OpenCodeWindows = NewArray;
		frmCode = new fCode();
		frmCode.Show();
	}

	static void RemoveEmptyFolders()
	{
		for(int i = 0; i < frmMain.tvwIndex.GetNodeCount(false); i++)
			for(int c = 0; c < frmMain.tvwIndex.Nodes[i].GetNodeCount(false); c++)
			{	
				if(frmMain.tvwIndex.Nodes[i].Nodes[c].GetNodeCount(true) == 0)
				{
					//EmptyNodes[EmptyNodes.GetUpperBound(0)] = frmMain.tvwIndex.Nodes[i].Nodes[c];
					//IncreaseNodeArray();
				}
			}
		foreach(TreeNode tempNode in EmptyNodes)
			frmMain.tvwIndex.Nodes.Remove(tempNode);
	}
	static void AddToPreviousSearch(CodeSample tempCode)
	{
		CodeSample[] NewArray;
		NewArray = new CodeSample[PreviousSearch.Length + 1];
		PreviousSearch.CopyTo(NewArray,0);
		NewArray[NewArray.Length - 1] = tempCode;
		PreviousSearch = NewArray;
	}
	static void LoadSystemTrayIcon()
	{
		Tray.Icon = frmMain.Icon;
		Tray.Text = "Library";
		Tray.Visible = true;
		Tray.MouseUp += new MouseEventHandler(TrayMouseUp);
	}
	static void TrayMouseUp(object sender, MouseEventArgs ex)
	{
		if(ex.Button == MouseButtons.Right)
		{
			frmMain.Visible = false;
			Finish = true;
		}
		else
		{
			frmMain.Visible = !frmMain.Visible;
		}
	}
	static void SetupIcons()
	{
		string[] Files = Directory.GetFiles(Application.StartupPath + @"\Icons\");
		Icons = new string[Files.Length];
		iconCount = -1;
		iconList.Images.Clear();
		AddIcon("Open.ico");
		AddIcon("Close.ico");
		frmMain.tvwIndex.ImageList = iconList;
	}
	static void LoadAppSettings()
	{
		Settings.Load("Library.xml");
		XmlNode root = Settings.ChildNodes[1]["Settings"];
		AppName = root["AppName"].InnerText;
		frmPos = new Point(Convert.ToInt32(root["X"].InnerText), Convert.ToInt32(root["Y"].InnerText));
		ShowEmptyFolders = Convert.ToBoolean(root["ShowEmptyFolders"].InnerText);
		SearchedFor = root["SearchedFor"].InnerText.Split('|');
	}

	static void SaveAppSettings()
	{
		XmlNode root = Settings.ChildNodes[1]["Settings"];
		if(frmMain.Text != AppName){root["AppName"].InnerText = AppName;};
		if(frmMain.Location != frmPos)
		{
			root["X"].InnerText = frmMain.Location.X.ToString();
			root["Y"].InnerText = frmMain.Location.Y.ToString();
		}
		root["ShowEmptyFolders"].InnerText = ShowEmptyFolders.ToString();
		root["SearchedFor"].InnerText = SaveSearched;
		Settings.Save("Library.xml");
	}

	static void LoadLanguages()
	{
		frmMain.cboFilter.Items.Clear();
		frmMain.tvwIndex.Nodes.Clear();
		int i = -1;
		XmlNode root = Settings.ChildNodes[1]["Languages"];
		Languages = new Language[root.ChildNodes.Count];
		frmMain.cboFilter.Items.Add("(no filter)");
		foreach(XmlNode tempNode in root)
		{
			i++;
			TreeNode treeNode = new TreeNode();
			Languages[i].Alias = tempNode["Alias"].InnerText;
			Languages[i].Name = treeNode.Text = tempNode["Name"].InnerText;
			Languages[i].Colour = treeNode.ForeColor = Color.FromName(tempNode["Color"].InnerText);
			Languages[i].FileName = tempNode["File"].InnerText;
			Languages[i].File = new DataSet();
			Languages[i].File.ReadXml(tempNode["File"].InnerText);
			Languages[i].Icon = treeNode.ImageIndex = treeNode.SelectedImageIndex = AddIcon(tempNode["Icon"].InnerText);
			treeNode.Tag = tempNode["Alias"].InnerText;
			frmMain.cboFilter.Items.Add(Languages[i].Name);
			frmMain.tvwIndex.Nodes.Add(treeNode);
		}
	}
	static void LoadTopics()
	{
		int i = -1;
		XmlNode root = Settings.ChildNodes[1]["Topics"];
		Topics = new Topic[root.ChildNodes.Count];
		foreach(XmlNode tempNode in root)
		{
		int d = GetLangNode(tempNode["Name"].InnerText);
			i++;
			Topics[i].Alias = tempNode["Alias"].InnerText;
			Topics[i].Name = tempNode["Name"].InnerText;
			Topics[i].Icon = AddIcon(tempNode["Icon"].InnerText);
			Topics[i].Samples = new CodeSample[0];
			Topics[i].CurrNum = 0;
			int Icon = GetIcon("Close.ico");
			for(int c = 0; c < Languages.Length; c++)
			{
				TreeNode treeNode = new TreeNode();
				treeNode.Tag = Languages[c].Alias;
				treeNode.Text = Topics[i].Name;
				treeNode.SelectedImageIndex = treeNode.ImageIndex = Icon;
				frmMain.tvwIndex.Nodes[c].Nodes.Add(treeNode);
			}
		}
	}
	static int GetLangNode(string strLang)
	{
	int i = -1;
		foreach(TreeNode temp in frmMain.tvwIndex.Nodes)
		{
			i++;
			if(temp.Text == strLang)
				return i;
		}
		i = -1;
		return i;
	}
	static void LoadCode()
	{
		int pTopic;
		int NumLang = -1;
		foreach(Language pLang in Languages)
		{
			int NumTopic = -1;
			NumLang++;
			DataRowCollection RowSet = pLang.File.Tables["Sample"].Rows;
			foreach(DataRow tempRow in RowSet)
			{
				NumTopic++;
				CodeSample tempCode;
				pTopic = GetTopic(tempRow["Topic"].ToString());
				CodeSample[] tempSamples = new CodeSample[Topics[pTopic].Samples.Length + 1];
				Topics[pTopic].Samples.CopyTo(tempSamples, 0);
				
				tempCode.Code = tempRow["Code"].ToString();
				tempCode.Search = tempRow["Search"].ToString();
				tempCode.Title = tempRow["Title"].ToString();
				tempCode.Topic = tempRow["Topic"].ToString();
				tempCode.Language = pLang.Name;
				tempCode.Usage = tempRow["Usage"].ToString();
				tempCode.Description = tempRow["Description"].ToString();
				tempSamples[tempSamples.Length - 1] = tempCode;
				Topics[pTopic].Samples = tempSamples;
				
				TreeNode treeNode = new TreeNode(Topics[pTopic].Samples[Topics[pTopic].Samples.Length - 1].Title, Topics[pTopic].Icon, Topics[pTopic].Icon);
				treeNode.Tag = Topics[pTopic].Samples.Length - 1;
				frmMain.tvwIndex.Nodes[GetLangNode(pLang.Name)].Nodes[GetTopic(tempCode.Topic)].Nodes.Add(treeNode);
				Topics[pTopic].CurrNum++;
			}
		}
		foreach(TreeNode lNode in frmMain.tvwIndex.Nodes)
			foreach(TreeNode tNode in lNode.Nodes){}
				//tNode.Text = tNode.Text + " (" + tNode.GetNodeCount(true).ToString() + ")";
	}
	public static int GetIcon(string strFile)
	{
		int c = - 1;
		for(int i = 0; i < Icons.Length; i++)
		{
			if(Icons[i] == strFile)
				c = i;
		}
		return c;
	}
	public static int GetTopic(string strAlias)
	{
		Topic fTopic = new Topic();
		for(int i = 0; i < Topics.Length; i++)
		{
			if(Topics[i].Alias == strAlias){return i;};
		}
		return -1;
	}
	static int AddIcon(string strFile)
	{
		try
		{
			for(int c = 0; c < Icons.Length; c++)
				if(Icons[c] == strFile)
					return c;
			Icon tempIcon = new Icon(Application.StartupPath + @"\Icons\" + strFile);
			iconCount++;
			Icons[iconCount] = strFile;
			iconList.Images.Add(tempIcon.ToBitmap(), Color.Black);
			return iconCount;
		}
		catch{return 0;}
	}

	public static void Search(string strTerm, string strFilter)
	{
		bool Found = false;
		AddSearchTerm(strTerm);
		if(frmMain.chkPrevResults.Checked == true)
		{
			for(int i = 0; i < PreviousSearch.Length; i++)
			{
				Found = false;
				string temp = PreviousSearch[i].Title.ToLower();
				if(temp.IndexOf(strTerm.ToLower(), 0) >= 0)
					Found = true;
				if(frmMain.chkTitles.Checked == false)
				{
					temp = PreviousSearch[i].Code.ToLower();
					if(temp.IndexOf(strTerm.ToLower(), 0) >= 0)
						Found = true;
					temp = PreviousSearch[i].Search.ToLower();
					if(temp.IndexOf(strTerm.ToLower(), 0) >= 0)
						Found = true;
				}
				
				if(Found == false)
				{
					PreviousSearch[i].Title = "Removed";
				}
			}
		}
		else
		{
			PreviousSearch = new CodeSample[0];
			foreach(Topic tempTopic in Topics)
			{
				foreach(CodeSample tempCode in tempTopic.Samples)
				{
					Found = false;
					if(tempCode.Language == strFilter || strFilter == "(no filter)")
					{
						string temp = tempCode.Title.ToLower();
						if(temp.IndexOf(strTerm.ToLower(), 0) >= 0)
							Found = true;
						temp = tempCode.Code.ToLower();
						if(frmMain.chkTitles.Checked == false)
						{
							if(temp.IndexOf(strTerm.ToLower(), 0) >= 0)
								Found = true;
							temp = tempCode.Search.ToLower();
							if(temp.IndexOf(strTerm.ToLower(), 0) >= 0)
								Found = true;
						}
					}
					if(Found == true)
					{
						AddToPreviousSearch(tempCode);
					}
					Found = false;
				}
			}
		}
		frmSearch = new fSearch();
		frmSearch.Show();
	}
	static void AddSearchTerm(string strTerm)
	{
	bool Found = false;
		foreach(object str in frmMain.cboSearch.Items)
		{
			if(str.ToString() == strTerm)
				Found = true;
		}
		if(Found == false)
			frmMain.cboSearch.Items.Add(strTerm);
	}
	public static CodeSample FindCode(string strTag, string strTopic)
	{
	CodeSample tempCode = new CodeSample();
		foreach(Topic top in Topics)
		{
			if(top.Name == strTopic)
				return top.Samples[Convert.ToInt32(strTag)];
		}
		return tempCode;
	}
	public static CodeSample FindCodeSearch(string strTitle, string strTopic)
	{
		CodeSample tempCode = new CodeSample();
		foreach(Topic top in Topics)
		{
			if(top.Name == strTopic)
				foreach(CodeSample i in top.Samples)
					if(i.Title == strTitle)
						return i;
		}
		return tempCode;
}
	public static void RemoveFromOpenWindowsList(string strText)
	{
	string[] NewArray;
	int d = 0;
		NewArray = new string[OpenCodeWindows.Length -1];
		for(int c = 0; c < NewArray.Length; c++)
		{
			if(OpenCodeWindows[c] != strText)
			{
				NewArray[c] = OpenCodeWindows[d];
				d++;	
			}
		}
		OpenCodeWindows = NewArray;
	}
	public static void UpdateEntry(CodeSample temp, CodeSample Original)
	{
		foreach(Language tempLang in Languages)
		{
			if(tempLang.Name == temp.Language)
				foreach(DataRow tempRow in tempLang.File.Tables["Sample"].Rows)
					if(tempRow["Title"].ToString() == Original.Title)
					{
						tempRow["Title"] = temp.Title;
						tempRow["Code"] = temp.Code;
						tempRow["Search"] = temp.Search;
						tempRow["Usage"] = temp.Usage;
						tempRow["Topic"] = GetTopicAlias(temp.Topic);
						tempRow.AcceptChanges();
						tempLang.File.Tables["Sample"].AcceptChanges();
						tempLang.File.AcceptChanges();
						tempLang.File.WriteXml(tempLang.FileName);
						SetupIcons();
						LoadLanguages();
						LoadTopics();
						LoadCode();
						return;
					}
		}
	}
	public static void DeleteEntry(CodeSample Original)
	{
		foreach(Language tempLang in Languages)
		{
			if(tempLang.Name == Original.Language)
				foreach(DataRow tempRow in tempLang.File.Tables["Sample"].Rows)
					if(tempRow["Title"].ToString() == Original.Title)
					{
						tempRow.Delete();
						tempLang.File.Tables["Sample"].AcceptChanges();
						tempLang.File.AcceptChanges();
						tempLang.File.WriteXml(tempLang.FileName);
						SetupIcons();
						LoadLanguages();
						LoadTopics();
						LoadCode();
						return;
					}
		}
	}
	static string GetTopicAlias(string strName)
	{
	string tempStr = "temp";
		foreach(Topic temp in Topics)
		{
			if(temp.Name == strName)
				return temp.Alias;
		}
		return tempStr;
	}
	public static void AddEntry(CodeSample temp)
	{
		foreach(Language pLang in Languages)
			if(pLang.Name == temp.Language)
			{
				DataRow tempRow = pLang.File.Tables["Sample"].NewRow();
				tempRow["Title"] = temp.Title;
				tempRow["Code"] = temp.Code;
				tempRow["Description"] = temp.Description;
				tempRow["Search"] = temp.Search;
				tempRow["Topic"] = GetTopicAlias(temp.Topic);
				tempRow["Usage"] = temp.Usage;
				pLang.File.Tables["Sample"].Rows.Add(tempRow);
				tempRow.AcceptChanges();
				pLang.File.Tables["Sample"].AcceptChanges();
				pLang.File.AcceptChanges();
				pLang.File.WriteXml(pLang.FileName);
				SetupIcons();
				LoadLanguages();
				LoadTopics();
				LoadCode();
			}
	}
}